#!/usr/bin/perl -w

# Copyright (C) 2010 Julien VAUBOURG
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

use CGI;
use CGI::Carp qw(fatalsToBrowser);
use lib '/home/sonde/classes';
use PSM::Zones;

# Creation du CGI
$cgi = new CGI;
$cgi->charset('utf-8');
print $cgi->header();

# Variables
my $template;
my $dernierId;
my $page;

# Recuperation du template
open(TEMPLATE, '../tpl/main.tpl') or die "ERREUR: Impossible d'ouvrir main.tpl\n";
$template .= $_ while <TEMPLATE>;
close TEMPLATE;


####
#### ENREGISTREMENT DES DONNEES
####

# Ecriture du nouveau fichier gzones.hash si des donnees ont ete envoyees
if($ENV{CONTENT_LENGTH} != 0) {
	my %zonesGZones;

	# Pour chaque champ envoye en POST, par ordre numerique des zones
	foreach (sort { int substr($a, 5) <=> int substr($b, 5) } $cgi->param()) {

		# Si c'est le champ d'une zone qui compose une GZone, on ajoute a la liste de zones de cette GZone
		if($_ =~ m/^GZone([0-9]+)_zone$/) {
			push(@{%zonesGZones->{$1}}, $cgi->param($_));
		}
	}

	my $gzonesHash = "\%Zones::GZones = (\n";

	# Pour chacune des listes de zone des GZones
	foreach (sort keys %zonesGZones) {

		# Creation d'une entree dans le hash de GZones, en specifiant le nom de la zone
		$gzonesHash .= '"'.$_.'" => ["'.$cgi->param("GZone$_").'", ["'.join('","', @{%zonesGZones->{$_}}).'"]],'."\n";
	}

	$gzonesHash .= ');';

	# Ecriture du nouveau gzones.hash
	open(GZONES, ">$RRD::root/config/gzones.hash") or die "Impossible d'ouvrir $RRD::root/config/gzones.hash\n";
	print GZONES $gzonesHash;
	close(GZONES);

	# Interpretation des nouvelles GZones pour la page en cours
	Zones::lire();
}


####
#### AFFICHAGE DE LA PAGE
####

$page .= "<form action='gzones.cgi' method='post' id='gzones'>"
	."<h2>Définitions des groupes de zones</h2>";

$page .= "<ul id='lesGZones'>";

# Pour chaque GZone
foreach my $id (sort { $a <=> $b } keys %Zones::GZones) {

	# On s'interesse qu'aux GZones qui n'ont pas ete generees automatiquement
	if($id =~ /^[0-9]+$/) {

		# Tableau de la GZone (son nom et les zones qu'elle regroupe)
		my @GZone = @{%Zones::GZones->{$id}};

		# Le tableau des zones qu'elle regroupe
		my @zones = @{@GZone[1]};

		# Son nom
		my $GZone = @GZone[0];

		$page  .= "<li>"

			# Nom de la GZone, modifiable et possibilite de supprimer toute la GZone
			."	<input type='text' name='GZone$id' value='$GZone' class='champNomZone' /><input type='button' value='X' title='Supprimer cette GZone' onclick='Zones.supprimer.apply(this)' class='boutonSup' />"
			."	<ul id='GZone$id'>";

		# Pour chaque zone de la GZone
		foreach (@zones) {
			my $options;

			$page  .= "	<li><select name='GZone${id}_zone'>";

			# Creation d'un select contenant toutes les zones, positionne sur la zone en cours
			foreach my $idZone (sort { Zones::label($a) cmp Zones::label($b) } keys(%Zones::zones)) {
				my $zone = %Zones::zones->{$idZone};
				$options .= "	<option value='$idZone'".($idZone eq $_ ? " selected='selected'" : '').">@{$zone}[0]</option>";
			}

			$options .= "		<option value='internet'".($_ eq 'internet' ? " selected='selected'" : '').">Internet</option>";
			$options = "		<option value='$_' class='attention'>???</option>$options" if $options !~ /selected=/;

			# Bouton supprimer, pour cette zone
			$page .= "$options</select><input type='button' value='X' title='Supprimer' onclick='Zones.supprimer.apply(this)' class='boutonSup' /></li>";
		}

		# Select et bouton pour ajouter une nouvelle zone à la GZone en cours
		$page  .= "		<li><select class='champAjout'>"
			. "			<option>-</option>";

		foreach my $idZone (sort { Zones::label($a) cmp Zones::label($b) } keys(%Zones::zones)) {
			my $zone = %Zones::zones->{$idZone};
			$page .= "		<option value='$idZone'>@{$zone}[0]</option>";
		}

		$page .= "			<option value='internet'>Internet</option>"
			."		</select><input type='button' value='Ajouter' onclick='GZones.ajouterZone.apply(this)' class='boutonAjout' /></li>"
			."	</ul>"
			."</li>";

		$dernierId = $id;
	}
}

# Champ et bouton pour ajouter une nouvelle GZone
$page .= "	<li><input type='text' class='champAjout ajoutAttente' value='Nouveau groupe' onfocus='Zones.libelleAjout(\"GZone\", 1, this)' onblur='Zones.libelleAjout(\"GZone\", 0, this)' /><input type='button' value='Ajouter' onclick='GZones.ajouterGZone.apply(this)' class='boutonAjout' /></li>"
	."</ul>";

# Liste des zones, pour les selects dynamiques
$page .= "<select class='champAjout' id='listeDesZones'>";
foreach my $idZone (sort { Zones::label($a) cmp Zones::label($b) } keys(%Zones::zones)) {
	my $zone = %Zones::zones->{$idZone};
	$page .= "<option value='$idZone'>@{$zone}[0]</option>";
}
$page  .= "	<option value='internet'>Internet</option>"
	. "</select>"

	# Dernier identifiant de GZone cree
	."<input type='hidden' id='dernierId' value='$dernierId' />"

	# Enregistrement des modifications
	."<div class='boutons'>"
	."	<input type='submit' value='Enregistrer' class='boutonEnregistrer' />"
	."</div>"
	."</form>";

# Bandeau patientez
$page .= "<div id='patientez'>Patientez</div>";

$template =~ s/<TITLE>/Gestion des groupes de zones/;
$template =~ s/<ONLOAD>//;
$template =~ s/<PAGE>/$page/;
print $template;
