/*
 * Copyright (C) 2010 Julien VAUBOURG
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

Zones = {

	// Ajoute une adresse dans la liste d'une zone, par l'intermediaire du champ d'ajout situe a la fin de cette zone
	// REM: Dans le contexte d'un bouton d'ajout d'adresse
	ajouterAdresse: function() {
		var champAdresse = this.previousSibling;

		if(champAdresse.value != '' && champAdresse.value != 'Nouvelle adresse') {
			var liste = this.parentNode.parentNode;
			var zone = liste.id;

			// Verification de l'unicite de la zone : avorte la fonction en cas de reussite de la fonction !
			if(Zones.ceNomExiste(champAdresse.value, liste)) {
				alert('Cette adresse a d\351j\340 \351t\351 ajout\351 \340 cette liste.');
				return false;
			}

			// Creation des elements a ajouter
			var nouvelleLigne = document.createElement('li');
			var nouveauChamp = document.createElement('input');
			var nouveauBoutonSup = document.createElement('input');

			// Nouveau champ de la liste d'adresses
			nouveauChamp.type = 'text';
			nouveauChamp.name = zone + '_adresse';
			nouveauChamp.value = champAdresse.value;

			// Nouveau bouton de suppression associe
			nouveauBoutonSup.type = 'button';
			nouveauBoutonSup.value = 'X';
			nouveauBoutonSup.title = 'Supprimer';
			nouveauBoutonSup.onclick = Zones.supprimer;
			nouveauBoutonSup.className = 'boutonSup';

			// Imbrication des elements
			nouvelleLigne.appendChild(nouveauChamp);
			nouvelleLigne.appendChild(nouveauBoutonSup);
	
			// Ajout du tout sur la page
			liste.insertBefore(nouvelleLigne, this.parentNode);

			// Reinitialisation du champ d'ajout d'adresse
			champAdresse.value = '';
			champAdresse.onblur();
		}
	},

	// Affichage un petit bandeau demandant de patienter avant d'executer la fonction demandee
	patientez: function(fonction) {
		document.getElementById('patientez').style.display = 'inline';

		// Ce 20ieme de seconde est necessaire pour que le bandeau ai le tps de s'afficher
		setTimeout(fonction + '; document.getElementById("patientez").style.display = "none"', 50);
	},

	// Mise a jour de la liste des IP restantes selon celles qui ont ete declarees dans les zones
	majListeIP: function() {

		// Tous les champs de textes
		var champs = document.getElementById('lesZones').getElementsByTagName('INPUT');

		// Liste des adresses IP
		var ips = document.getElementById('listeip').getElementsByTagName('SPAN');

		// Pour chaque IP
		for(var i = 0; i < ips.length; i++) {

			// L'IP correspond a une sous-reseau des zones
			var trouve = false;

			// Pour chaque champ de texte
			for(var c = 0; c < champs.length; c++) {

				// Si ce champ correspond a un champ d'adresse
				if(/zone[0-9]+_adresse/.test(champs[c].name)) {

					// Si l'IP en cours de la liste est dans le sous-reseau du champ en cours 
					if(Zones.estDansSousReseau(champs[c].value, ips[i].innerHTML)) {
						trouve = true;
						break;
					}
				}
			}

			// L'IP est cachee si elle a ete reperee dans les zones au moins une fois
			ips[i].style.display = trouve ? 'none' : 'block';
		}
	},

	// Une IP donnee appartient-elle au sous-reseau ?
	// @arg : subnet - Sous-reseau avec son masque en octal (ex: 192.168.107.0/255.255.255.0) ou adresse IP
	// @arg : ip - Adresse IP
	estDansSousReseau: function(subnet, ip) {

		// Le sous-reseau est bien un sous-reseau et pas une simple IP
		if(/\//.test(subnet)) {

			// Separation du sous-reseau et de son masque
			var netmask = subnet.split(/\//)[1];
			subnet = subnet.split(/\//)[0];

			// Separation des octets de l'IP, le subnet et le masque
			ip = ip.split(/\./);
			subnet = subnet.split(/\./);
			netmask = netmask.split(/\./);

			// Pour chacun des 4 octets de chaque
			for(var i = 0; i < 4; i++) {

				// ET logique entre l'octet de l'IP et l'octet du masque
				var AND = ip[i] & netmask[i];

				// Si le resultat du ET logique ne correspond pas a l'octet du sous-reseau c'est que l'IP n'appartient pas au sous-reseau
				if(AND != subnet[i])
					return false;
			}

			// Si la fonction n'a toujours pas avortee, c'est que tous les octets verifient que l'IP
			// appartient au sous-reseau donne
			return true;
		}

		// On compare deux IP : simple comparaison
		return subnet == ip;
	},

	// Ajoute une zone dans la liste des zones, par l'intermediaire du champ d'ajout situe a la fin de cette liste
	// REM: Dans le contexte d'un bouton d'ajout de zone
	ajouterZone: function() {
		var champAdresse = this.previousSibling;

		if(champAdresse.value != '' && champAdresse.value != 'Nouvelle zone') {
			var liste = this.parentNode.parentNode;

			// Verification de l'unicite de la zone : avorte la fonction en cas de reussite de la fonction !
			if(Zones.ceNomExiste(champAdresse.value, liste)) {
				alert('Impossible de cr\351er la zone :\nce nom a d\351j\340 \351t\351 attribu\351.');
				return false;
			}

			var nouvelleLigne = document.createElement('li');
			var nouveauChamp = document.createElement('input');

			// Creation des elements a ajouter
			var nouvelleListeAdresses = document.createElement('ul');
			var nouvelleLigneAdresse = document.createElement('li');
			var nouveauBoutonSup = document.createElement('input');
			var nouveauChampAjout = document.createElement('input');
			var nouveauBoutonAjout = document.createElement('input');

			// Definition du numero de la nouvelle zone
			nouvelleListeAdresses.id = 'zone' + (++document.getElementById('dernierId').value);

			// Nouveau champ contenant le nom de la zone
			nouveauChamp.type = 'text';
			nouveauChamp.name = nouvelleListeAdresses.id;
			nouveauChamp.value = champAdresse.value;
			nouveauChamp.className = 'champNomZone';

			// Nouveau bouton de suppression pour la nouvelle zone
			nouveauBoutonSup.type = 'button';
			nouveauBoutonSup.value = 'X';
			nouveauBoutonSup.title = 'Supprimer cette zone';
			nouveauBoutonSup.onclick = Zones.supprimer;
			nouveauBoutonSup.className = 'boutonSup'

			// Nouveau champ d'ajout d'ajout d'adresse pour la nouvelle zone
			nouveauChampAjout.type = 'text';
			nouveauChampAjout.onfocus = function() { Zones.libelleAjout('adresse', 1, this) };
			nouveauChampAjout.onblur = function() {
				Zones.convertirMasque.apply(this);
				Zones.libelleAjout('adresse', 0, this);
			};
			nouveauChampAjout.value = 'Nouvelle adresse';
			nouveauChampAjout.className = 'champAjout ajoutAttente';

			// Nouveau bouton d'ajout pour la nouvelle zone
			nouveauBoutonAjout.type = 'button';
			nouveauBoutonAjout.onclick = Zones.ajouterAdresse;
			nouveauBoutonAjout.value = 'Ajouter';
			nouveauBoutonAjout.className = 'boutonAjout';

			// Imbrication des elements
			nouvelleLigne.appendChild(nouveauChamp);
			nouvelleLigne.appendChild(nouveauBoutonSup);
			nouvelleLigneAdresse.appendChild(nouveauChampAjout);
			nouvelleLigneAdresse.appendChild(nouveauBoutonAjout);
			nouvelleListeAdresses.appendChild(nouvelleLigneAdresse);
			nouvelleLigne.appendChild(nouvelleListeAdresses);
	
			// Ajout du tout sur la page
			liste.insertBefore(nouvelleLigne, this.parentNode);

			// Reinitialisation du champ d'ajout de zone
			champAdresse.value = '';
			champAdresse.onblur();
		}
	},

	// Verifie si un nom de zone a deja ete donne
	// @arg : nom - Nom de la zone/adresse  comparer
	// @arg : liste - Liste des zones/adresses
	// @return : Booleen (existe deja ou non)
	ceNomExiste: function(nom, liste) {
		var dernierChamp = Noeuds.dernierNoeud.apply(liste);

		for(var i = 0; i < liste.childNodes.length; i++) {

			// Avorte la fonction si on est arrive au dernier champ de la liste (le champ d'ajout)
			if(liste.childNodes[i] == dernierChamp)
				return false;

			if(!/^#/.test(liste.childNodes[i].nodeName)) {
				var champNom = Noeuds.premierNoeud.apply(liste.childNodes[i]);

				if(new RegExp('^' + RegExp.escape(nom) + '$', 'gi').test(champNom.value)) {
					champNom.id = (champNom.id != '') ? champNom.id : 'idTmp';
					champNom.style.border = '4px solid red';
					setTimeout('var elmt = document.getElementById("' + champNom.id + '"); elmt.style.border = 0; if(elmt.id == "idTmp") elmt.removeAttribute("id");', 2100);

					return true;
				}
			}
		}

		// La zone/adresse est unique
		return false;
	},

	// Supprime un adresse ou une zone complete
	// REM: Dans le contexte d'un bouton de suppression d'adresse ou de zone
	supprimer: function() {
		this.parentNode.parentNode.removeChild(this.parentNode);
	},

	// Gere les evenements d'entree et sorties des champs d'ajout pour changer le texte et la couleur
	// @arg : type - zone ou adresse
	// @arg : action - 1 ou 0 (respectivement onfocus et onblur)
	// @arg : champ - Champ d'ajout
	libelleAjout: function(type, action, champ) {
		if(action == 1 && /ajoutAttente/.test(champ.className)) { // onfocus
			champ.className = champ.className.replace(/ ajoutAttente/, '');
			champ.value = '';
		} else if(action == 0 && champ.value == '') { // onblur
			champ.className += ' ajoutAttente';
			champ.value = 'Nouvelle ' + type;
		}
	},

	// Remplace une adresse de type 192.168.107.* en 192.168.107.0/24
	// TEM: Dans le contexte d'un champ
	convertirMasque: function() {
		this.value = this.value.replace(/\.\*$/, '.0/255.255.255.0');
	}
}


///
/// METHODE VOLEE
/// http://simonwillison.net/2006/Jan/20/escape/
///

// Protege tous les caracteres speciaux des regexp dans une chaine donnee
// @arg : text - Chaine a proteger
// @return : Chaine protegee
RegExp.escape = function(text) {
	if(!arguments.callee.sRE) {
		var specials = ['/', '.', '*', '+', '?', '|', '(', ')', '[', ']', '{', '}', '\\'];
		arguments.callee.sRE = new RegExp('(\\' + specials.join('|\\') + ')', 'g');
	}

	return text.replace(arguments.callee.sRE, '\\$1');
}
