# Copyright (C) 2010 Julien VAUBOURG
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

use strict;
package CategoriesInternet;


# class CategoriesInternet
#
# UTILISATION
#
# Cette classe est utilisee pour faire l'analyse du trafic internet des sites à risque transmis a une zone ou une gzone.
# Les sites à risque (critiques) sont definis grace a des listes de categories definies par des noms de domaines, jugees appartenant
# a la categorie. Ces listes sont recuperees sur le site de l'universite de Toulouse.
#
# Pour comparer les adresses IP des sites visites avec ces noms de domaine, on fait une resolution DNS de toutes les IP appartenant
# a la zone internet, et on la stocke dans un fichier config/nomsServeurs.hash (fonction nomsServeurs de Capture.pm). La fonction
# genererZonesCategories de la classe presente cree ensuite une zone par categorie ayant repertoriee des sites lui appartenant, dans
# les noms de domaine des IP internet. Cette comparaison se fait sur la base du nombre de parties des noms de domaines des listes de
# categories (ex: pour google.com dans une categorie, on utilisera que les deux dernieres parties de lists.google.com pour comparer :
# ce nom de serveur appartient donc a la categorie qui reference google.com). A noter qu'on compare donc des noms de serveurs (obtenus
# avec la resolution DNS) avec les noms de domaine des listes de categorie : cette methode ne peut fonctionner que pour les plus
# gros sites.
#
# Si des adresses IP sont directement renseignees dans les categories, on compare d'IP a IP.


###
### VARIABLES DE CLASSE
###

our %categories; # Mise en cache des fichiers de domaines du repertoire categories/ (syntaxe: nomCategorie => [domaine1, domaine2])
our %nomsServeurs; # Mise en cache du hash d'association entre les IP Internet et leur nom de serveur ('ip' => 'nomServeur')


###
### METHODES DE CLASSE
###

# Lit le fichier config/nomsServeurs.hash et met en cache les associations IPInternet->NomServeur dans %nomsServeurs
sub miseEnCacheNomsServeurs {
	my $lignes;

	# Ouverture et parcours du .hash
	open(SERVEURS, "$RRD::root/config/nomsServeurs.hash") or die "ERREUR: Impossible d'ouvrir $RRD::root/config/nomsServeurs.hash";
	$lignes .= $_ while <SERVEURS>;

	# Prise en compte du code perl du fichier, dans le script
	eval $lignes;

	close SERVEURS;
}

# Lit les fichiers de domaines du repertoire categories/ et les met en cache dans %categories en indiquant chaque fois le nom de la categorie
# comme cle et un tableau compose des domaines du fichier correspondant comme valeur
sub miseEnCacheDesCategories {

	# Ouverture du dossier contenant les fichiers de definition des categories
	opendir(CATEGORIES, "$RRD::root/categories") or die "ERREUR: Impossible d'ouvrir $RRD::root/categories/";

	# Pour chacun de ces fichiers
	foreach my $fichier (readdir CATEGORIES) {

		# Ouverture du fichier
		open(CATEGORIE, "$RRD::root/categories/$fichier") or die "ERREUR: Impossible d'ouvrir $RRD::root/categories/$fichier";

		# Pour chaque domaine du fichier
		while(<CATEGORIE>) {
			chomp($_);

			# On l'ajoute ÃÂ  un tableau indexe dans %categories avec le nom du fichier
			push(@{%CategoriesInternet::categories->{$fichier}}, $_);
		}
	}
}

# Recherche les IP Internet pour lesquelles le nom du serveur semble correspondre au domaine d'une des categories, et les classes dans la zone
# portant le nom de la categorie correspondante (prefixee par CAT_).
sub genererZonesCategories {

	my @categories = keys %CategoriesInternet::categories;

	# Pour chaque IP Internet trouvee
	foreach my $ip (@{Zones::listerIP('internet')}) {
		my $attribue = 0;

		# Recuperation du nom de serveur de l'IP, si il existe
		my $nomServeur = %CategoriesInternet::nomsServeurs->{$ip};
		my $isIP = !defined $nomServeur;
		$nomServeur = $ip if $isIP;

		# Pour chaque categorie
		FORCAT: for(my $i = 0; $i < @categories && !$attribue; $i++) {
			my $categorie = @categories[$i];
			my $nomsServeurs = %CategoriesInternet::categories->{$categorie};

			my ($GZoneCategorie) = $categorie =~ /^niv([0-9]+)_/;
			$categorie =~ s/^niv([0-9]+)_//;

			# Pour chaque nom de serveur de la categorie
			FORSERVCAT: for(my $j = 0; $j < @$nomsServeurs-1 && !$attribue; $j++) {

				# Si le nom du serveur en cours de la categorie est une adresse IP
				if(@$nomsServeurs[$j] =~ /[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+/) {

					# Si le nom de serveur qu'on recherche n'est pas une adresse IP, on ne cherche meme pas a comparer
					next FORSERVCAT if !$isIP;
				# Le nom du serveur en cours de la categorie n'est pas une IP
				} else {

					# Du coup, si le nom du serveur qu'on recherche est une IP, on passe a l'iteration suivante
					next FORSERVCAT if $isIP;

					# Decoupe des deux noms de serveurs en elements du nom (separes initialement par les points)
					my @nomServeurMorceaux = split(/\./, $nomServeur);
					my @nomServeurCatMorceaux = split(/\./, @$nomsServeurs[$j]);

					# Si le nom de serveur qu'on recherche a plus d'elements que le nom de serveur en cours de la categorie,
					# alors on ne garde que les derniers morceaux du serveur qu'on recherche pour comparer
					if(@nomServeurMorceaux > @nomServeurCatMorceaux) {
						$nomServeur = pop @nomServeurMorceaux;
						$nomServeur = pop(@nomServeurMorceaux).'.'.$nomServeur for 1..@nomServeurCatMorceaux-1;

					# Si c'est strictement l'inverse, on considere que ca n'est pas possible donc on ne compare meme pas
					} elsif (@nomServeurMorceaux < @nomServeurCatMorceaux) {
						next FORSERVCAT;
					}
				}

				# Si le domaine correspond au nom de serveur (ou l'IP ÃÂ  l'IP),
				# on ajoute l'IP Internet a la zone de la categorie
				if(@$nomsServeurs[$j] eq $nomServeur) {
					$attribue = 1;

					# Creation ou mise a jour de la zone
					if(!defined %Zones::zones->{'CAT_'.$categorie}) {
						%Zones::zones->{'CAT_'.$categorie} = [$categorie, [$ip]];
					} else {
						push(@{@{%Zones::zones->{'CAT_'.$categorie}}[1]}, $ip);
						push(@{%Zones::zonesIP->{'CAT_'.$categorie}}, $ip);
					}

					# Creation eventuelle de la GZone
					if(!defined %Zones::GZones->{'GCAT_'.$GZoneCategorie}) {
						%Zones::GZones->{'GCAT_'.$GZoneCategorie} = [$GZoneCategorie, ["CAT_$categorie"]];

					# Mise a jour
					} elsif(!grep($_ eq "CAT_$categorie", @{@{%Zones::GZones->{'GCAT_'.$GZoneCategorie}}[1]})) {
						push(@{@{%Zones::GZones->{'GCAT_'.$GZoneCategorie}}[1]}, "CAT_$categorie");
					}

					# Mise a jour du nom de serveur, en ne gardant que la partie qui a servie a comparer
					%CategoriesInternet::nomsServeurs->{$ip} = $nomServeur;

					# Inutile de chercher une autre categorie a ce nom de serveur
					last FORCAT;
				}
			}
		}
	}
}

# Retourne des informations en LaTeX concernant la visite des categories depuis une zone donnee
# @arg : type - Type d'affichage souhaite
# 			-> topCategoriesParGCategorie
# 			-> topServeursParCategorie
# 			-> topGeneralParServeur
# 			-> camembertParGCategories
# @arg : limite - N'afficher que les n resultats les plus importants
# @arg : zone - Zone ou GZone a observer
# @return : Code LaTeX
sub toLaTeX {
	my ($type, $zone, $limite) = @_;

	return topCategoriesParGCategorie($zone, $limite) if $type eq 'topCategoriesParGCategorie';
	return topServeursParCategorie($zone, $limite) if $type eq 'topServeursParCategorie';
	return topGeneralParServeur($zone, $limite) if $type eq 'topGeneralParServeur';
	return camembertParGCategories($zone, $limite) if $type eq 'camembertParGCategories';

	return "ERREUR: Type inexistant\n";
}

# Retourne le code LaTeX d'un camembert representant la repartition du trafic constate entre la zone et les differentes GCategories
# @arg : zone - Zone ou GZone a observer
# @arg : limite - N'afficher que les n categories les plus visitees
# @return : Code LaTeX
sub camembertParGCategories {
	my ($zone, $limite) = @_;
	my @GCategories;

	# Pour chaque GZone
	foreach (sort keys %Zones::GZones) {
		if($_ =~ /^GCAT_/) {

			# Si c'est une GCategorie, ajouter le flux entre la zone donee et cette GCategorie
			my $req = Zones::requete("GZONE:$_", $zone, 'PROTO_*.rrd');
			push(@GCategories, new GroupeRRD($req, Zones::label($_), 1)) if @$req != 0;
		}
	}

	# Rassemblement des flux zone-GCategorie dans un GroupeRRD afin de les representer dans un camembert
	if(@GCategories != 0) {
		my $repartition = new GroupeRRD(\@GCategories, 'Categories', 0, $limite);

		# Creation du camembert
		return $repartition->toLaTeX('camembert');
	}

	return 0;
}

# Retourne le code LaTeX des tableaux des categories les plus visitees de chaque GCategorie
# @arg : zone - Zone ou GZone a observer
# @arg : limite : N'afficher que les n categories les plus visites pour chaque GCategorie
# @return : Code LaTeX
sub topCategoriesParGCategorie {
	my ($zone, $limite) = @_;
	my @GCategories;
	my $retour;

	# Pour chaque GZone
	foreach (sort keys %Zones::GZones) {

		# Ajouter la GZone au tableau si c'est une GCategorie
		if($_ =~ /^GCAT_/) {
			my @categories;

			# Pour chaque categorie (zone) de la GCategorie (GZone)
			foreach my $uneZone (@{@{%Zones::GZones->{$_}}[1]}) {

				# Ajouter un GroupeRRD du trafic entre la zone a observer et la zone courante
				my $req = Zones::requete($uneZone, $zone, 'PROTO_*.rrd');
				push(@categories, new GroupeRRD($req, Zones::label($uneZone), 1)) if @$req != 0;
			}

			# Un GroupeRRD permet de rassembler les flux pour les trier a nouveau et en afficher un tableau
			if(@categories != 0) {
				my $parCategorie = new GroupeRRD(\@categories, Zones::label($_));
				$retour .= $parCategorie->toLaTeX('tableau');
			}
		}
	}

	return $retour;
}

# Retourne le code LaTeX des serveurs les plus visites par categorie
sub topServeursParCategorie {
        my ($zone, $limite) = @_;
        my $retour;

	# Pour chaque GZone (GCategorie)
	foreach my $GCategorie (sort keys %Zones::GZones) {
		if($GCategorie =~ /^GCAT_/) {

			# Pour chaque zone (categorie) de la GCategorie
			foreach (@{@{%Zones::GZones->{$GCategorie}}[1]}) {
				$GCategorie =~ s/^GCAT_//;

				# Creation du flux entre la zone donnee et la zone en cours
				my $req = Zones::requete($_, $zone, 'PROTO_*.rrd');

				if(@$req != 0) {
					my $parServeur = new GroupeRRD($req, "Niveau ${GCategorie}_".Zones::label($_), 0);

					# Creation du tableau des serveurs les plus visites pour cette categorie
					rassemblerParServeur($parServeur, $limite);
					$retour .= $parServeur->toLaTeX('tableau');
				}
			}
		}
	}

	return $retour;
}

# Retourne le code LaTeX d'un tableau des noms de serveurs les plus visites, toutes categories confondues
# @arg : limite - Ne garder que les n premier serveurs les plus consultes
# @return : Code LaTeX
sub topGeneralParServeur {
	my ($zone, $limite) = @_;
	my @serveurs;
	my $toutesZones;

	# Pour chaque zone
	foreach (sort keys %Zones::zones) {

		# Si c'est une zone de categorie
		if($_ =~ /^CAT_/) {

			# Recuperation des donnees entre la categorie (adresses internet) et la zone a observer
			my $categorie = new GroupeRRD(Zones::requete($_, $zone, 'PROTO_*.rrd'), Zones::label($_), 1);

			# Recuperation des sources que le GroupeRRD a cree
			@serveurs = (@serveurs, @{$categorie->{source}}) if $categorie;
		}
	}

	# Creation d'un groupe a partir des sources de tous les groupes crees, afin d'avoir tous les serveurs melanges
	my $topServeurs = new GroupeRRD(\@serveurs, 'Toutes catégories', 0);

	if($topServeurs) {

		# Rassemblement des serveurs par nom de domaine (tous les *.google.com ensemble par ex)
		rassemblerParServeur($topServeurs, $limite);

		return $topServeurs->toLaTeX('tableau');
	}
}

# Fusion de toutes les IP qui ont un nom de serveur identique (sur la base des serveurs des categories)
# et attribution des noms de serveurs dans les labels
# @arg : groupeRRD - GroupeRRD dont les labels sont a transformer en nom de serveur en rassemblant les identiques
# @arg : limite - Nombre maximum de valeurs a garder, une fois que les serveurs ont ete fusionnes
sub rassemblerParServeur {
	my ($groupeRRD, $limite) = @_;

	my @parServeurLabels;
	my %parServeurTotaux;

	# Pour chaque label du GroupeRRD (IP)
	foreach (@{$groupeRRD->{labels}}) {

		# Recherche du nom de serveur correspond ÃÂ  l'IP en cours
		my $nomServeur = %CategoriesInternet::nomsServeurs->{$_};

		# On garde l'IP si elle n'a pas trouve de domaine correspondant
		$nomServeur = $_ if !defined $nomServeur;

		# Ajout du nom de serveur dans les nouveaux totaux : si une IP a deja eu le meme nom de serveur,
		# les totaux seront donc additionnes
		%parServeurTotaux->{$nomServeur} += $groupeRRD->{totaux}->{$_};
	}

	# Creation du nouveau tableau de labels qui servira a definir l'ordre des valeurs (du plus grand au plus petit).
	# On attribue donc le tableau des labels (noms de serveur), en prenant soin de les trier en fonction des totaux
	# qui leur sont associes
	my @parServeurLabels = sort { %parServeurTotaux->{$b} <=> %parServeurTotaux->{$a} } keys %parServeurTotaux;

	# Si il y a plus de labels que la limite ne le permet, on supprime tout ce qui depasse
	if(@parServeurLabels > $limite) {
		for(my $i=@parServeurLabels-1; $i > $limite-1; $i--) {
			delete @parServeurLabels[$i];
		}
	}

	# Mises a jour des labels et totaux du GroupeRRD avec les nouveaux tableaux (passage de IP a noms de serveurs)
	$groupeRRD->{labels} = \@parServeurLabels;
	$groupeRRD->{totaux} = \%parServeurTotaux;
}

1; # Obligatoire
