# Copyright (C) 2010 Julien VAUBOURG
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

use strict;
use PSM::RRD;
package LaTeX;


# class LaTeX
#
# UTILISATION
#
# PSM::LaTeX est utilisee uniquement pour lire le fichier template du rapport latex, remplacer les <TAG> par les informations
# correspondantes et ecrire le rapport final dans un fichier .tex.
# La methode add permet de renseigner petit a petit le corps-meme du rapport stocke dans la variable d'instance <rapport> et de
# l'ajouter dans le rapport final en remplacement du tag <DOC>, lors de l'ecriture du fichier.
# Le fichier peut ensuite etre compile en PDF.


###
### VARIABLES DE CLASSE
###

our $template = 'rapport.tpl'; # Nom du fichier de template
our $client; # Nom du client a afficher sur le rapport
our $intro; # Introduction au rapport

lireInfos();


###
### CONSTRUCTEUR
###

# @arg : fichier - Nom du fichier TeX a produire
sub new {
	my ($classe, $fichier) = @_;

	# VARIABLES D'INSTANCE
	my $this = { 'fichier' => $fichier };

	# Instanciation
	bless ($this, $classe);

	# Retour de la reference
	return $this;
}


###
### METHODES DE CLASSE
###

# Recupere les dates de debut et de fin de la periode d'observation des RRD, ainsi que le nom du client
# et l'introduction du rapport
sub lireInfos {
	open(INFOS, "$RRD::root/config/infos") or die "ERREUR: Impossible d'ouvrir $RRD::root/config/infos";

	my $i = 0;
	while(<INFOS>) {
		chomp($_);
		$RRD::debut = $_ if $i == 0;
		$RRD::fin = $_ if $i == 1;
		$LaTeX::client = $_ if $i == 2;
		$LaTeX::intro = $_ if $i > 2;
		$i++;
	}

	close INFOS;
	RRD::setPeriode($RRD::debut, $RRD::fin);
}


###
### METHODES D'INSTANCE
###

# Ajouter un morceau de LaTeX au corps du rapport
# @arg : ajout - Code LaTeX
sub add {
	my ($this, $ajout) = @_;
	$this->{rapport} .= $ajout;
}

# Ecrire le preambyle du fichier LaTeX
sub ecrirePreambule {
	my $this = shift;

	# Lecture et recuperation du template 
	open(TEMPLATE, "$RRD::root/rapport/$LaTeX::template") or die "ERREUR: Impossible d'ouvrir $RRD::root/rapport/$LaTeX::template";
	my $rapport;
	$rapport .= $_ while <TEMPLATE>;
	close TEMPLATE;

	# Dates de debut et de fin d'analyse, au format francais et sans l'heure
	my $debut = RRD::jour($RRD::debut);
	my $fin = RRD::jour($RRD::fin);

	# Remplacement des tag dans le template
	$rapport =~ s/<QUI>/$LaTeX::client/g;
	$rapport =~ s/<DEBUT>/$debut/g;
	$rapport =~ s/<FIN>/$fin/g;
	$rapport =~ s/<INTRO>/$LaTeX::intro/;

	# Ecriture du fichier .tex
	open(RAPPORT, ">$RRD::root/rapport/$this->{fichier}") or die "ERREUR: Impossible d'ecrire $RRD::root/rapport/$this->{fichier}\n";
	print RAPPORT $rapport;
	close RAPPORT;
}

# Completer le rapport avec la mise en memoire
sub ecrire {
	my $this = shift;

	open(RAPPORT, ">>$RRD::root/rapport/$this->{fichier}") or die "ERREUR: Impossible d'ecrire $RRD::root/rapport/$this->{fichier}\n";
	print RAPPORT $this->{rapport};
	close RAPPORT;

	$this->{rapport} = '';
}

# Cloture du LaTeX dans le fichier de rapport
sub finaliser {
	my $this = shift;

	$this->add('\end{document}');
	$this->ecrire();
}

# Compilation du .tex de ecrire() en .pdf
sub compiler {
	my $this = shift;

	# Ecriture du .tex si il n'existe pas
	ecrire($this) if !-f "$RRD::root/rapport/$this->{fichier}";

	# Changement de dossier pour que le PDF soit genere au meme niveau que le .tex
	chdir "$RRD::root/rapport";

	# Compilation du .tex en .pdf avec pdflatex
	# Des fichiers .toc .log et .aux vont etre crees en plus du .pdf
	# Deux compilations sont necessaires pour la table des matieres et le nombre de pages !
	`pdflatex $this->{fichier}`;
}

1; # Obligatoire
